/*
TSC - A program to convert Truevision Targa (TGA) and bitmap (BMP) images to data for use in TI assembly or C programming
Copyright (C) 2007  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "main.h"


int main(int argc, char **argv) {
	FILE *src;
	FILE *dest=NULL;
	int h=0;
	int w=0;
	int f=0;
	int value=0;
	int i;
	unsigned char *ptr;
	char optbuf[20];
	char iprgm[18];
	options opts={0,0,0,0,0,0,0,50};
	char *image_bname;
	IMAGE img;

	
	for(h=1;(h<argc && argv[h][0]=='-');h++) {
		strncpy(optbuf,argv[h],2);
		optbuf[2]=0;
		
		if(!strcmp(argv[h],"-s2"))
			opts.shades=2;
		else if(!strcmp(argv[h],"-s4"))
			opts.shades=4;
		else if(!strcmp(argv[h],"-s7"))
			opts.shades=7;
		else if(!strcmp(argv[h],"-s8"))
			opts.shades=8;
		else if(!strcmp(argv[h],"-oh"))
			opts.output|=OUT_HEX;
		else if(!strcmp(argv[h],"-la"))
			opts.output|=LANG_ASM;
		else if(!strcmp(argv[h],"-lr"))
			opts.output|=LANG_RAW;
		else if(!strcmp(argv[h],"-dh"))
			opts.output|=DISP_HORIZ;
		else if(!strcmp(argv[h],"-ap"))
			opts.output|=ARRAY_PROD;
		else if(!strcmp(argv[h],"-os"))
			opts.output|=OUT_SINGLE;
		else if(!strcmp(argv[h],"-i"))
			opts.interlaced=1;
		else if(!strcmp(argv[h],"-v"))
			opts.verbosity=1;
		else if(!strcmp(optbuf,"-b")) {
			strcpy(optbuf,argv[h]+2);
			sscanf(optbuf,"%d",&opts.brightness);
		}
		else if(!strcmp(optbuf,"-c")) {
			strcpy(optbuf,argv[h]+2);
			sscanf(optbuf,"%d",&opts.contrast);
		}
		else if(!strcmp(optbuf,"-p")) {
			strncpy(optbuf,argv[h]+2,strlen(argv[h]+1)>17?17:strlen(argv[h]+1));
			
			if(strlen(argv[h])>2) //if a program was specified
				strcpy(iprgm,optbuf);
			else
				strcpy(iprgm,IMAGE_VIEWER);
			opts.preview=1;
		}
			
		else if(!strcmp(argv[h],"-h") || !strcmp(argv[h],"--help")) {
			fprintf(stderr,"-----------------\nTSC Sprite Converter\nBy Peter Fernandes\n-----------------\n\ntsc [options] infile [infile] ...\n\nConverts TGA or BMP images to data for use in TI assembly or C programming.\nOptions:\n-h       Show this help message\n-s[2478] Output data for 2,4,7 or 8 shade grayscale\n-oh      Set output type to hex (outputs in binary by default)\n-l[ar]   Set output format to _A_ssembly data or _R_aw ASCII text (outputs in C\n         arrays by default)\n-ap      Write C array sizes as a product of height and width for widths >32\n-dh      Write binary data horizontally instead of in rows\n-os      Write all data to one file (writes to separate files for each input by\n         default)\n-b[X]    Adjust brightness of image by X, from -50 to 50\n-c[X]    Adjust contrast of image by X, from 0 to 100\n-i       Interlace grayscale output data\n-v       Make program verbose in output\n-p[prgm] Show a preview of the image after conversion is done using prgm\n");
			return 0;
		}
	}
	
	if(!opts.shades)
		opts.shades=4;
	if(!(opts.output&OUT_HEX))
		opts.output|=OUT_BIN;
	if(!(opts.output&LANG_C) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
		opts.output|=LANG_C;
	if(!(opts.verbosity))
		opts.verbosity=0;
	if(!(opts.brightness))
		opts.brightness=0;

	
	if(h==argc) {
		fprintf(stderr,"Error: No input files\n");
		return -1;
	}
	
	for(i=h;i<argc;i++) {
			
		if(!(src=fopen(argv[i],"rb"))) {
			fprintf(stderr,"Error opening source file \"%s\"\n",argv[i]);
			continue;
		}
		
		
		
		image_bname=basename(argv[i]);

		
		image_bname=realloc(image_bname,strlen(image_bname)+5);
		strcat(image_bname,".txt");
		
		if(!(opts.output&OUT_SINGLE) || i==h) {
			if(!(opts.output&OUT_SINGLE)) {
				if(!(dest=fopen(image_bname,"wb"))) {
					fprintf(stderr,"Error opening destination file\n");
					return -1;
				}
			}
			else {
				if(!(dest=fopen("sprite.txt","wb"))) {
					fprintf(stderr,"Error opening destination file\n");
					return -1;
				}
			}
		}
		
		
		
		if(opts.verbosity) {
			image_bname=basename(argv[i]);
			if(image_bname) {
				fprintf(stderr,"Processing image \"%s\" with %d colors...",image_bname,opts.shades);
				if(!(opts.output&OUT_SINGLE))
					free(image_bname);
			}
		}
		
		if(opts.output&OUT_SINGLE) {
			*(image_bname+strlen(image_bname)-4)=0;
			fprintf(dest,"%s:\n",image_bname);
			free(image_bname);
		}
		
		
		//load the image data
		if((h=load_image(&img,src,&opts))) {
			if(opts.verbosity)
				fprintf(stderr,"Failed: ");
				
			
			switch (h) {
				case -2:
					fprintf(stderr,"Image must be 24-bit\n");
					break;
				case -3:
					fprintf(stderr,"Memory\n");
					break;
				case -4:
					fprintf(stderr,"Image width must be a multiple of 8\n");
					break;
			}
			
			continue;
		}
		
		
		
		
		if(opts.shades<7) {
			if(!opts.interlaced) {	
				if(opts.output&LANG_C) {
					switch (img.width) {
						case 8:
							fprintf(dest,"unsigned char sprite_l[%d]={",img.height);
							break;
						case 16:
							fprintf(dest,"unsigned short sprite_l[%d]={",img.height);
							break;
						case 32:
							fprintf(dest,"unsigned long sprite_l[%d]={",img.height);
							break;
					}
					
					if(img.width>32) {
						if(opts.output&ARRAY_PROD)
							fprintf(dest,"unsigned char sprite_l[%d*%d]={",img.height,(img.width/8));
						else
							fprintf(dest,"unsigned char sprite_l[%d]={",img.height*(img.width/8));
					}
				}
				else if(opts.output&LANG_ASM) {
					switch (img.width) {
						case 8:
							fprintf(dest,"sprite_l:\n\t.byte ");
							break;
						case 16:
							fprintf(dest,"sprite_l:\n\t.word ");
							break;
						case 32:
							fprintf(dest,"sprite_l:\n\t.long ");
							break;
					}
					
					if(img.width>32)
						fprintf(dest,"sprite_l:\n\t.byte ");
				
				}
				else { //raw output
					if(opts.shades==2)
						fprintf(dest,"Sprite Data:\n");
					else
						fprintf(dest,"Light Data:\n");
				}
				
				
				ptr=img.data;
				
				//write light data
				for(h=0;h<img.height;h++) {
					if((h<img.height && h) && img.width<=32) {
						if(opts.output&LANG_RAW)
							fputc(' ',dest);
						else
							fputc(',',dest);
					}
					
					if(opts.output&OUT_BIN) {
						if(opts.output&DISP_HORIZ)
							fputs("0b",dest);
						else
							fputs("\n\t0b",dest);
					}
		
					
					for(w=0,value=0;w<img.width;w++,ptr++) {
						if(opts.output&OUT_BIN)
							fputc((*ptr==LIGHT || *ptr==BLACK)?'1':'0',dest);
						else if(opts.output&OUT_HEX)
							value|=((*ptr==LIGHT || *ptr==BLACK)?1:0)<<(img.width<=32?(img.width-1-w):(img.width-1-w)&7);
						
						if(img.width>32 && !((w+1)%8) && ((h>=img.height-1)?(w<img.width-1):1) && w && (opts.output&OUT_BIN)) {
							if(opts.output&LANG_RAW)
								fputc(' ',dest);
							else
								fputc(',',dest);
								
							if(w<img.width-1)
								fputs("0b",dest);	
						}
						else if(img.width>32 && !((w+1)%8) && (opts.output&OUT_HEX)) {
							fprintf(dest,"0x%.2X",value);
							if((h>=img.height-1)?(w<img.width-1):1) {
								if(opts.output&LANG_RAW)
									fputc(' ',dest);
								else
									fputc(',',dest);
							}
						}
					}
					
					if((opts.output&OUT_HEX) && img.width<=32) {
						switch (img.width) {
							case 8:
								fprintf(dest,"0x%.2X",value);
								break;
							case 16:
								fprintf(dest,"0x%.4X",value);
								break;
							case 32:
								fprintf(dest,"0x%.8X",value);
								break;
						}
					}
						
				}
				
				if((opts.output&OUT_BIN) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW)) {
					if(!(opts.output&DISP_HORIZ))
						fputs("\n};\n\n",dest);
					else
						fputs("};\n\n",dest);
				}
				else if((opts.output&OUT_HEX) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
					fputs("};\n\n",dest);
				else
					fputc('\n',dest);
			
				
				
				if(opts.shades==4) {
					if(opts.output&LANG_C) {
						switch (img.width) {
							case 8:
								fprintf(dest,"unsigned char sprite_d[%d]={",img.height);
								break;
							case 16:
								fprintf(dest,"unsigned short sprite_d[%d]={",img.height);
								break;
							case 32:
								fprintf(dest,"unsigned long sprite_d[%d]={",img.height);
								break;
						}
						
						if(img.width>32) {
							if(opts.output&ARRAY_PROD)
								fprintf(dest,"unsigned char sprite_l[%d*%d]={",img.height,(img.width/8));
							else
								fprintf(dest,"unsigned char sprite_l[%d]={",img.height*(img.width/8));
						}
					}
					else if(opts.output&LANG_ASM) {
						switch (img.width) {
							case 8:
								fprintf(dest,"sprite_d:\n\t.byte ");
								break;
							case 16:
								fprintf(dest,"sprite_d:\n\t.word ");
								break;
							case 32:
								fprintf(dest,"sprite_d:\n\t.long ");
								break;
						}
						
						if(img.width>32)
							fprintf(dest,"sprite_d:\n\t.byte ");
					}
					else { //raw output
						fprintf(dest,"Dark Data:\n");
					}
					
					ptr=img.data;
					
					//write dark data
					for(h=0;h<img.height;h++) {
						if((h<img.height && h) && img.width<=32) {
							if(opts.output&LANG_RAW)
								fputc(' ',dest);
							else
								fputc(',',dest);
						}
						
						if(opts.output&OUT_BIN) {
							if(opts.output&DISP_HORIZ)
								fputs("0b",dest);
							else
								fputs("\n\t0b",dest);
						}
			
						
						for(w=0,value=0;w<img.width;w++,ptr++) {
							if(opts.output&OUT_BIN)
								fputc((*ptr==DARK || *ptr==BLACK)?'1':'0',dest);
							else if(opts.output&OUT_HEX)
								value|=((*ptr==DARK || *ptr==BLACK)?1:0)<<(img.width<=32?(img.width-1-w):(img.width-1-w)&7);
							
							if(img.width>32 && !((w+1)%8) && ((h>=img.height-1)?(w<img.width-1):1) && w && (opts.output&OUT_BIN)) {
								if(opts.output&LANG_RAW)
									fputc(' ',dest);
								else
									fputc(',',dest);
									
								if(w<img.width-1)
									fputs("0b",dest);	
							}
							else if(img.width>32 && !((w+1)%8) && (opts.output&OUT_HEX)) {
								fprintf(dest,"0x%.2X",value);
								if((h>=img.height-1)?(w<img.width-1):1) {
									if(opts.output&LANG_RAW)
										fputc(' ',dest);
									else
										fputc(',',dest);
								}
							}
						}
						
						if((opts.output&OUT_HEX) && img.width<=32) {
							switch (img.width) {
								case 8:
									fprintf(dest,"0x%.2X",value);
									break;
								case 16:
									fprintf(dest,"0x%.4X",value);
									break;
								case 32:
									fprintf(dest,"0x%.8X",value);
									break;
							}
						}
					}
					
					if((opts.output&OUT_BIN) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW)) {
						if(!(opts.output&DISP_HORIZ))
							fputs("\n};\n\n",dest);
						else
							fputs("};\n\n",dest);
					}
					else if((opts.output&OUT_HEX) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
						fputs("};\n\n",dest);
					else
						fputc('\n',dest);
				}
			}
			else {
				if(opts.output&LANG_C) {
					switch (img.width) {
						case 8:
							fprintf(dest,"unsigned char sprite[%d]={",img.height*2);
							break;
						case 16:
							fprintf(dest,"unsigned short sprite[%d]={",img.height*2);
							break;
						case 32:
							fprintf(dest,"unsigned long sprite[%d]={",img.height*2);
							break;
					}
					
					if(img.width>32) {
						if(opts.output&ARRAY_PROD)
							fprintf(dest,"unsigned char sprite[%d*%d]={",img.height*2,(img.width/8));
						else
							fprintf(dest,"unsigned char sprite[%d]={",img.height*2*(img.width/8));
					}
				}
				else if(opts.output&LANG_ASM) {
					switch (img.width) {
						case 8:
							fprintf(dest,"sprite:\n\t.byte ");
							break;
						case 16:
							fprintf(dest,"sprite:\n\t.word ");
							break;
						case 32:
							fprintf(dest,"sprite:\n\t.long ");
							break;
					}
					
					if(img.width>32)
						fprintf(dest,"sprite:\n\t.byte ");
				
				}
				else if(opts.shades==2) { //raw output
					fprintf(dest,"Sprite Data:\n");
				}
				
				
				ptr=img.data;
				
				//write interlaced data
				for(h=0;h<img.height;h++) {
					if(opts.output&OUT_BIN) {
						if(opts.output&DISP_HORIZ)
							fputs("0b",dest);
						else
							fputs("\n\t0b",dest);
					}
		
					
					for(f=0;f<2;f++) {
						for(w=0,value=0;w<img.width;w++,ptr++) {
							if(opts.output&OUT_BIN)
								fputc(((!f?(*ptr==LIGHT):(*ptr==DARK)) || *ptr==BLACK)?'1':'0',dest);
							else if(opts.output&OUT_HEX)
								value|=(((!f?(*ptr==LIGHT):(*ptr==DARK)) || *ptr==BLACK)?1:0)<<(img.width<=32?(img.width-1-w):(img.width-1-w)&7);
							
							if(img.width>32 && !((w+1)%8) && ((h>=img.height-1)?(w<img.width-1):1) && w && (opts.output&OUT_BIN)) {
								if(opts.output&LANG_RAW)
									fputc(' ',dest);
								else
									fputc(',',dest);
									
								if(w<img.width-1)
									fputs("0b",dest);	
							}
							else if(img.width>32 && !((w+1)%8) && (opts.output&OUT_HEX)) {
								fprintf(dest,"0x%.2X",value);
								if((h>=img.height-1)?(w<img.width-1):1) {
									if(opts.output&LANG_RAW)
										fputc(' ',dest);
									else
										fputc(',',dest);
								}
							}
							else if(img.width<=32 && (w>=img.width-1) && (h<(img.height-(opts.output&OUT_BIN?1:0)) || !f) && (opts.output&OUT_HEX?(h || f):1)) {
								if(opts.output&LANG_RAW)
									fputc(' ',dest);
								else
									fputc(',',dest);
									
								if(w<img.width-1) {
									if(opts.output&OUT_BIN)
										fputs("0b",dest);
									else
										fputs("0x",dest);
								}
							
							}
						}
						
						if(!f)
							ptr-=img.width; //go back and reread for other plane
						
						if((opts.output&OUT_HEX) && img.width<=32) {
							switch (img.width) {
								case 8:
									fprintf(dest,"0x%.2X",value);
									break;
								case 16:
									fprintf(dest,"0x%.4X",value);
									break;
								case 32:
									fprintf(dest,"0x%.8X",value);
									break;
							}
						}
					}						
				}
				
				if((opts.output&OUT_BIN) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW)) {
					if(!(opts.output&DISP_HORIZ))
						fputs("\n};\n\n",dest);
					else
						fputs("};\n\n",dest);
				}
				else if((opts.output&OUT_HEX) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
					fputs("};\n\n",dest);
				else
					fputc('\n',dest);
			}
		}
		
		/////////////////////////////////
		//////__3-PLANE__////////////////
		/////////////////////////////////
		
		else if(opts.shades>=7) {
			if(opts.output&LANG_C) {
				switch (img.width) {
					case 8:
						fprintf(dest,"unsigned char sprite_p1[%d]={",img.height);
						break;
					case 16:
						fprintf(dest,"unsigned short sprite_p1[%d]={",img.height);
						break;
					case 32:
						fprintf(dest,"unsigned long sprite_p1[%d]={",img.height);
						break;
				}
				
				if(img.width>32) {
					if(opts.output&ARRAY_PROD)
						fprintf(dest,"unsigned char sprite_p1[%d*%d]={",img.height,(img.width/8));
					else
						fprintf(dest,"unsigned char sprite_p1[%d]={",img.height*(img.width/8));
				}
			}
			else if(opts.output&LANG_ASM) {
				switch (img.width) {
					case 8:
						fprintf(dest,"sprite_p1:\n\t.byte ");
						break;
					case 16:
						fprintf(dest,"sprite_p1:\n\t.word ");
						break;
					case 32:
						fprintf(dest,"sprite_p1:\n\t.long ");
						break;
				}
				
				if(img.width>32)
					fprintf(dest,"sprite_p1:\n\t.byte ");
			}
			else { //raw output
				fprintf(dest,"Plane 1 Data:");
			}
	
			
			ptr=img.data;
			
			//write data for plane 1
			for(h=0;h<img.height;h++) {
				if((h<img.height && h) && img.width<=32) {
					if(opts.output&LANG_RAW)
						fputc(' ',dest);
					else
						fputc(',',dest);
				}
				
				if(opts.output&OUT_BIN) {
					if(opts.output&DISP_HORIZ)
						fputs("0b",dest);
					else
						fputs("\n\t0b",dest);
				}
	
				
				for(w=0,value=0;w<img.width;w++,ptr++) {
					if(opts.output&OUT_BIN)
						fputc((*ptr==DARK_WHITE || *ptr==LIGHT || *ptr==DARK || *ptr==BLACK)?'1':'0',dest);
					else if(opts.output&OUT_HEX)
						value|=(((*ptr==DARK_WHITE || *ptr==LIGHT || *ptr==DARK || *ptr==BLACK)?1:0)<<(img.width<=32?(img.width-1-w):(img.width-1-w)&7));
					
					if(img.width>32 && !((w+1)%8) && ((h>=img.height-1)?(w<img.width-1):1) && w && (opts.output&OUT_BIN)) {
						if(opts.output&LANG_RAW)
							fputc(' ',dest);
						else
							fputc(',',dest);
						if(w<img.width-1)
							fputs("0b",dest);	
					}
					else if(img.width>32 && !((w+1)%8) && (opts.output&OUT_HEX)) {
						fprintf(dest,"0x%.2X",value);
						if((h>=img.height-1)?(w<img.width-1):1) {
							if(opts.output&LANG_RAW)
								fputc(' ',dest);
							else
								fputc(',',dest);
						}
					}
				}
				
				if((opts.output&OUT_HEX) && img.width<=32) {
					switch (img.width) {
						case 8:
							fprintf(dest,"0x%.2X",value);
							break;
						case 16:
							fprintf(dest,"0x%.4X",value);
							break;
						case 32:
							fprintf(dest,"0x%.8X",value);
							break;
					}
				}
			}
			
			if((opts.output&OUT_BIN) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW)) {
				if(!(opts.output&DISP_HORIZ))
					fputs("\n};\n\n",dest);
				else
					fputs("};\n\n",dest);
			}
			else if((opts.output&OUT_HEX) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
				fputs("};\n",dest);
			else
				fputc('\n',dest);
			
			if(opts.output&LANG_C) {
				switch (img.width) {
					case 8:
						fprintf(dest,"unsigned char sprite_p2[%d]={",img.height);
						break;
					case 16:
						fprintf(dest,"unsigned short sprite_p2[%d]={",img.height);
						break;
					case 32:
						fprintf(dest,"unsigned long sprite_p2[%d]={",img.height);
						break;
				}
				
				if(img.width>32) {
					if(opts.output&ARRAY_PROD)
						fprintf(dest,"unsigned char sprite_p2[%d*%d]={",img.height,(img.width/8));
					else
						fprintf(dest,"unsigned char sprite_p2[%d]={",img.height*(img.width/8));
				}
			}
			else if(opts.output&LANG_ASM) {
				switch (img.width) {
					case 8:
						fprintf(dest,"sprite_p2:\n\t.byte ");
						break;
					case 16:
						fprintf(dest,"sprite_p2:\n\t.word ");
						break;
					case 32:
						fprintf(dest,"sprite_p2:\n\t.long ");
						break;
				}
				
				if(img.width>32)
					fprintf(dest,"sprite_p2:\n\t.byte ");	
			}
			else { //raw output
				fprintf(dest,"\nPlane 2 Data:");
			}
			
			ptr=img.data;
			
			//write data for plane 2
			for(h=0;h<img.height;h++) {
				if((h<img.height && h) && img.width<=32) {
					if(opts.output&LANG_RAW)
						fputc(' ',dest);
					else
						fputc(',',dest);
				}
				
				if(opts.output&OUT_BIN) {
					if(opts.output&DISP_HORIZ)
						fputs("0b",dest);
					else
						fputs("\n\t0b",dest);
				}
				
				for(w=0,value=0;w<img.width;w++,ptr++) {
					if(opts.output&OUT_BIN)
						fputc((*ptr==LIGHT_LIGHT || *ptr==LIGHT || *ptr==LIGHT_BLACK || *ptr==BLACK)?'1':'0',dest);
					else if(opts.output&OUT_HEX)
						value|=(((*ptr==LIGHT_LIGHT || *ptr==LIGHT || *ptr==LIGHT_BLACK || *ptr==BLACK)?1:0)<<(img.width<=32?(img.width-1-w):(img.width-1-w)&7));
					
					if(img.width>32 && !((w+1)%8) && ((h>=img.height-1)?(w<img.width-1):1) && w && (opts.output&OUT_BIN)) {
						if(opts.output&LANG_RAW)
							fputc(' ',dest);
						else
							fputc(',',dest);
						if(w<img.width-1)
							fputs("0b",dest);	
					}
					else if(img.width>32 && !((w+1)%8) && (opts.output&OUT_HEX)) {
						fprintf(dest,"0x%.2X",value);
						if((h>=img.height-1)?(w<img.width-1):1) {
							if(opts.output&LANG_RAW)
								fputc(' ',dest);
							else
								fputc(',',dest);
						}
					}
				}
				
				if((opts.output&OUT_HEX) && img.width<=32) {
					switch (img.width) {
						case 8:
							fprintf(dest,"0x%.2X",value);
							break;
						case 16:
							fprintf(dest,"0x%.4X",value);
							break;
						case 32:
							fprintf(dest,"0x%.8X",value);
							break;
					}
				}
			}
	
	
			
			if((opts.output&OUT_BIN) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW)) {
				if(!(opts.output&DISP_HORIZ))
					fputs("\n};\n\n",dest);
				else
					fputs("};\n\n",dest);
			}
			else if((opts.output&OUT_HEX) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
				fputs("};\n",dest);
			else
				fputs("\n",dest);
			
			if(opts.output&LANG_C) {
				switch (img.width) {
					case 8:
						fprintf(dest,"unsigned char sprite_p3[%d]={",img.height);
						break;
					case 16:
						fprintf(dest,"unsigned short sprite_p3[%d]={",img.height);
						break;
					case 32:
						fprintf(dest,"unsigned long sprite_p3[%d]={",img.height);
						break;
				}
				
				if(img.width>32) {
					if(opts.output&ARRAY_PROD)
						fprintf(dest,"unsigned char sprite_p3[%d*%d]={",img.height,(img.width/8));
					else
						fprintf(dest,"unsigned char sprite_p3[%d]={",img.height*(img.width/8));
				}
			}
			else if(opts.output&LANG_ASM) {
				switch (img.width) {
					case 8:
						fprintf(dest,"sprite_p3:\n\t.byte ");
						break;
					case 16:
						fprintf(dest,"sprite_p3:\n\t.word ");
						break;
					case 32:
						fprintf(dest,"sprite_p3:\n\t.long ");
						break;
				}
						
				if(img.width>32)
					fprintf(dest,"sprite_p3:\n\t.byte ");
			}
			else { //raw output
				fprintf(dest,"\nPlane 3 Data:");
			}
			
			ptr=img.data;
			
			//write data for plane 3
			for(h=0;h<img.height;h++) {
				if((h<img.height && h) && img.width<=32) {
					if(opts.output&LANG_RAW)
						fputc(' ',dest);
					else
						fputc(',',dest);
				}
				
				if(opts.output&OUT_BIN) {
					if(opts.output&DISP_HORIZ)
						fputs("0b",dest);
					else
						fputs("\n\t0b",dest);
				}
	
				
				for(w=0,value=0;w<img.width;w++,ptr++) {
					if(opts.output&OUT_BIN)
						fputc(((*ptr==(opts.shades==8?LIGHT_DARK:LIGHT)) || *ptr==DARK || *ptr==LIGHT_BLACK || *ptr==BLACK)?'1':'0',dest);
					else if(opts.output&OUT_HEX)
						value|=((((*ptr==(opts.shades==8?LIGHT_DARK:LIGHT)) || *ptr==DARK || *ptr==LIGHT_BLACK || *ptr==BLACK)?1:0)<<(img.width<=32?(img.width-1-w):(img.width-1-w)&7));
					
					if(img.width>32 && !((w+1)%8) && ((h>=img.height-1)?(w<img.width-1):1) && w && (opts.output&OUT_BIN)) {
						if(opts.output&LANG_RAW)
							fputc(' ',dest);
						else
							fputc(',',dest);
						if(w<img.width-1)
							fputs("0b",dest);	
					}
					else if(img.width>32 && !((w+1)%8) && (opts.output&OUT_HEX)) {
						fprintf(dest,"0x%.2X",value);
						if((h>=img.height-1)?(w<img.width-1):1) {
							if(opts.output&LANG_RAW)
								fputc(' ',dest);
							else
								fputc(',',dest);
						}
					}
				}
				
				if((opts.output&OUT_HEX) && img.width<=32) {
					switch (img.width) {
						case 8:
							fprintf(dest,"0x%.2X",value);
							break;
						case 16:
							fprintf(dest,"0x%.4X",value);
							break;
						case 32:
							fprintf(dest,"0x%.8X",value);
							break;
					}
				}
			}
			
			if((opts.output&OUT_BIN) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW)) {
				if(!(opts.output&DISP_HORIZ))
					fputs("\n};\n\n",dest);
				else
					fputs("};\n\n",dest);
			}
			else if((opts.output&OUT_HEX) && !(opts.output&LANG_ASM) && !(opts.output&LANG_RAW))
				fputs("};\n",dest);
			else
				fputs("\n",dest);
		}
		
		if(opts.verbosity) {
			fprintf(stderr,"Done\n");
		}
	
		if(opts.preview) {
			if(write_preview(iprgm,img.data,img.height,img.width,opts.shades))
				fprintf(stderr,"Warning: Could not write preview file\n");
		}
		
		
		
		free(img.data);
		
		fclose(src);
		
		if(!(opts.output&OUT_SINGLE) || (h==argc-1))
			fclose(dest);
		else
			fprintf(dest,"\n\n");
	}
	
	return 0;
}

